(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2007-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: parser_plus.ml,v 1.4.2.1 2011-09-21 13:08:04 fclement Exp $ *)

open Parser;;

let (alphas, nums, notc) =
  let rec flipflop min max =
    if min = max then [min] else
    min :: (flipflop (min + 1) max) in
  ( List.map Char.chr ([95; 45] @ flipflop 97 122 @ flipflop 65 90)
  , List.map Char.chr (flipflop 48 57)
  , fun l -> List.filter (fun c -> not (List.mem c l))
      (List.map Char.chr (flipflop 0 255))
  )
;;
(* alphas is [a-zA-Z_-] *)
(* nums is [0-9] *)

let spaces = [ '\r'; '\n'; '\t'; ' '; ];;

let implode l =
  let rec implode i = function
  | c :: cs ->
    let s = implode (i + 1) cs in s.[i] <- c; s
  | [] -> String.create i in
  implode 0 l
;;

let ignore p = p >> return ();;

let p_letter = any (List.map char alphas);;
let p_digit = any (List.map char nums);;
let p_space = any (List.map char spaces);;

let p_ident = many1 p_letter >>= fun l -> return (implode l);;

let to_num l =
  let rec to_num = function
    | h :: t -> (Char.code h - Char.code '0') + to_num t * 10
    | [] -> 0 in
  to_num (List.rev l)
;;

let p_integer = many1 p_digit >>= fun l -> return (to_num l);;

let p_word s =
  let rec p_word i =
    if i = String.length s then return () else
    char s.[i] >> p_word (i + 1) in
  p_word 0
;;
